require 'rest-client'

require File.dirname(__FILE__) + '/services/authentication_service'
require File.dirname(__FILE__) + '/services/user_service'
require File.dirname(__FILE__) + '/services/widget_service'
require File.dirname(__FILE__) + '/services/user_widget_service'
require File.dirname(__FILE__) + '/domain/application_config'

#
# Interface for gem methods
#
class WidgetsApiClient

  #
  # Sets Application Configuration
  #
  # @param [String] client_id
  # @param [String] client_secret
  # @param [String] base_url
  #
  def self.set_config(client_id, client_secret, base_url)
    ApplicationConfig.set_config(client_id, client_secret, base_url)
  end

  #
  # Return Client Id
  #
  # @return [String]
  #
  def self.client_id
    ApplicationConfig.client_id
  end

  #
  # Creates authentication token
  #
  # @param [object] auth_data Authentication object
  #
  # @return [Json] Response with brearer and refresh token
  #
  def self.create_authentication_token(auth_data)
    AuthenticationService.create_authentication(auth_data)
  end

  #
  # Refresh authentication token
  #
  # @param [Object] auth_data Authentication object
  # @param [String] bearer_token
  #
  # @return [Json] Response with brearer and refresh token
  #
  def self.refresh_authentication_token(auth_data, bearer_token)
    AuthenticationService.refresh_authentication_token(auth_data, bearer_token)
  end

  #
  # Revokes authentication token
  #
  # @param [Object] auth_data Authentication object
  # @param [String] bearer_token
  #
  # @return [Json] Response with revoke message
  #
  def self.revoke_authentication_token(auth_data, bearer_token)
    AuthenticationService.revoke_authentication_token(auth_data, bearer_token)
  end

  #
  # Creates User
  #
  # @param [Object] user_data User Object for create
  #
  # @return [Json] Response of create user
  #
  def self.create_user(user_data)
    UserService.create_user(user_data)
  end

  #
  # Updates existing user
  #
  # @param [Object] user_data User Object for update
  # @param [String] bearer_token
  #
  # @return [Json] Response of update user
  #
  def self.update_user(user_data, bearer_token)
    UserService.update_user(user_data, bearer_token)
  end

  #
  # Verifies if email is already registered
  #
  # @param [Object] user_data user object conatining email
  #
  # @return [Json] Response of verify email
  #
  def self.check_email(user_data)
    UserService.check_email(user_data)
  end

  #
  # Sends email with reset password link
  #
  # @param [Object] user_data User object
  #
  # @return [Json] Response of reset password
  #
  def self.reset_password(user_data)
    UserService.reset_password(user_data)
  end

  #
  # Updates old password to new password
  #
  # @param [Object] user_data User object with old and new password details
  # @param [String] bearer_token
  #
  # @return [Json] Response of change password
  #
  def self.change_password(user_data, bearer_token)
    UserService.change_password(user_data, bearer_token)
  end

  #
  # Displays logged in user details
  #
  # @param [String] bearer_token
  #
  # @return [Json] Response containing logged in user details
  #
  def self.show_logged_in_user(bearer_token)
    UserService.show_logged_in_user(bearer_token)
  end

  #
  # Displays details of user id
  #
  # @param [String] user_id
  # @param [String] bearer_token
  #
  # @return [Json] Response containing details of given user id
  #
  def self.show_user_id(user_id, bearer_token)
    UserService.show_user_id(user_id, bearer_token)
  end

  #
  # Displays own widgets
  #
  # @param [String] bearer_token
  #
  # @return [Json] Response containing list of own widgets
  #
  def self.show_private_widgets(bearer_token)
    UserWidgetService.get_private_widgets(bearer_token)
  end

  #
  # Displays own widget with search term
  #
  # @param [String] search_term
  # @param [String] bearer_token
  #
  # @return [Json] Response containing list of own widget by search term
  #
  def self.show_private_widgets_with_search_term(search_term, bearer_token)
    UserWidgetService.get_private_widgets_with_search_term(search_term, bearer_token)
  end

  #
  # Displays widgets for given user id
  #
  # @param [String] user_id
  # @param [String] bearer_token
  #
  # @return [Json] Resopnse containing widgets list for given user id
  #
  def self.show_widgets_by_user_id(user_id, bearer_token)
    UserWidgetService.get_widgets_by_user_id(user_id, bearer_token)
  end

  #
  # Displays widgets for given user id and serach term
  #
  # @param [String] user_id
  # @param [String] search_term
  # @param [String] bearer_token
  #
  # @return [Json] Response contating widget details for user id and serach term
  #
  def self.show_widgets_by_user_id_with_search_term(user_id, search_term, bearer_token)
    UserWidgetService.get_widgets_by_user_id_with_search_term(user_id, search_term, bearer_token)
  end

  #
  # Creates Widget
  #
  # @param [Object] widget_data Widget object for create
  # @param [String] bearer_token
  #
  # @return [Json] Response of create widget
  #
  def self.create_widget(widget_data, bearer_token)
    WidgetService.create_widget(widget_data, bearer_token)
  end

  #
  # Updates existing widget
  #
  # @param [Object] widget_data Widget object for update
  # @param [String] widget_id widget to update
  # @param [String] bearer_token
  #
  # @return [Json] Response of update widget
  #
  def self.update_widget(widget_data, widget_id, bearer_token)
    WidgetService.update_widget(widget_data, widget_id, bearer_token)
  end

  #
  # Deletes own widget for given widget id
  #
  # @param [String] widget_id widgets to delete
  # @param [String] bearer_token
  #
  # @return [Json] Response of delete widget
  #
  def self.destroy_widget(widget_id, bearer_token)
    WidgetService.delete_widget(widget_id, bearer_token)
  end

  #
  # Lists own widgets
  #
  # @param [String] bearer_token
  #
  # @return [String] Response with list of own widgets
  #
  def self.get_private_widgets(bearer_token)
    WidgetService.get_own_widgets(bearer_token)
  end

  #
  # Lists public widgets
  #
  # @param [String] bearer_token
  #
  # @return [Json] Response with list of public visible widgets
  #
  def self.get_public_widgets(bearer_token)
    WidgetService.get_public_widgets(bearer_token)
  end

  #
  # Lists public widgets with search term
  #
  # @param [String] search_term
  # @param [String] bearer_token
  #
  # @return [Json] Response with searched public widgets by given search term
  #
  def self.get_public_widgets_with_search_term(search_term, bearer_token)
    WidgetService.get_public_widgets_with_search_term(search_term, bearer_token)
  end
end
